// Copyright © Microsoft Corporation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.opengroup.osdu.register.provider.azure.ddms;

import lombok.AllArgsConstructor;
import lombok.Getter;
import lombok.NoArgsConstructor;
import org.opengroup.osdu.register.ddms.model.Ddms;
import org.opengroup.osdu.register.ddms.model.RegisteredInterface;

import java.sql.Timestamp;
import java.util.Set;
import java.util.stream.Collectors;

@Getter
@AllArgsConstructor
@NoArgsConstructor
public class DdmsDoc {

    private String id;
    private String name;
    private String description;
    private String contactEmail;
    private Timestamp createdDateTimeEpoch;
    private String dataPartitionId;
    private Set<RegisteredInterfaceDoc> interfaces;

    public DdmsDoc(Ddms ddms, String dataPartitionId){
        this.id = ddms.getId();
        this.name = ddms.getName();
        this.description = ddms.getDescription();
        this.contactEmail = ddms.getContactEmail();
        this.createdDateTimeEpoch = new Timestamp(System.currentTimeMillis());
        this.dataPartitionId = dataPartitionId;
        this.interfaces = ddms.getInterfaces().stream().map(this::convertToRegisteredInterfaceDoc).collect(Collectors.toSet());
    }

    private RegisteredInterfaceDoc convertToRegisteredInterfaceDoc(RegisteredInterface item){
        return new RegisteredInterfaceDoc(item.getEntityType(), item.getSchema());
    }

}

