/*
  Copyright 2021 Google LLC
  Copyright 2021 EPAM Systems, Inc

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */

package org.opengroup.osdu.register.provider.reference.util;

import lombok.RequiredArgsConstructor;
import org.apache.http.HttpStatus;
import org.apache.http.impl.client.HttpClients;
import org.opengroup.osdu.core.common.model.http.AppException;
import org.opengroup.osdu.core.gcp.googleidtoken.GoogleIdTokenException;
import org.opengroup.osdu.core.gcp.googleidtoken.IGoogleIdTokenFactory;
import org.opengroup.osdu.register.utils.IGoogleServiceAccount;
import org.springframework.stereotype.Component;

@Component
@RequiredArgsConstructor
public class GoogleServiceAccountImpl implements IGoogleServiceAccount {

  private static final String INTERNAL_SERVER_ERROR_REASON = "Internal server error";

  private final IGoogleIdTokenFactory googleIdTokenFactory;

  @Override
  public String getIdToken(String keyString, String audience) {
    try {
      return this.googleIdTokenFactory.getGoogleIdToken(keyString, audience,
          HttpClients.createDefault());
    } catch (GoogleIdTokenException e) {
      throw new AppException(HttpStatus.SC_INTERNAL_SERVER_ERROR, INTERNAL_SERVER_ERROR_REASON,
          e.getMessage());
    }
  }

  @Override
  public String getPrivateKeyId(String keyString) {
    try {
      return this.googleIdTokenFactory.getPrivateKeyId(keyString);
    } catch (GoogleIdTokenException e) {
      throw new AppException(HttpStatus.SC_INTERNAL_SERVER_ERROR, INTERNAL_SERVER_ERROR_REASON,
          e.getMessage());
    }
  }
}
